library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity prng is
generic
(
	C_BASEADDR		: std_logic_vector	:= X"00000000";
	C_HIGHADDR		: std_logic_vector	:= X"0000000F";
	C_OPB_AWIDTH	: integer				:= 32;
	C_OPB_DWIDTH	: integer				:= 32;
	C_USER_ID_CODE	: integer				:= 3;
	C_FAMILY			: string					:= "virtex2p";
	TAP0_POS			: integer				:= 1;
	TAP1_POS			: integer				:= 2;
	TAP2_POS			: integer				:= 4;
	TAP3_POS			: integer				:= 15
);
port
(
	OPB_Clk						: in	std_logic;
	OPB_Rst						: in	std_logic;
	Sl_DBus						: out	std_logic_vector(0 to C_OPB_DWIDTH-1);
	Sl_errAck					: out	std_logic;
	Sl_retry						: out	std_logic;
	Sl_toutSup					: out	std_logic;
	Sl_xferAck					: out	std_logic;
	OPB_ABus						: in	std_logic_vector(0 to C_OPB_AWIDTH-1);
	OPB_BE						: in	std_logic_vector(0 to C_OPB_DWIDTH/8-1);
	OPB_DBus						: in	std_logic_vector(0 to C_OPB_DWIDTH-1);
	OPB_RNW						: in	std_logic;
	OPB_select					: in	std_logic;
	OPB_seqAddr					: in	std_logic
);

	attribute SIGIS						: string;
	attribute SIGIS of OPB_Clk			: signal is "Clk";
	attribute SIGIS of OPB_Rst			: signal is "Rst";

end entity prng;

------------------------------------------------------------------------------
-- Architecture section
------------------------------------------------------------------------------

architecture IMP of prng is

	signal data_BUS2IP	: std_logic_vector(31 downto 0);	-- Data from the Bus to this peripheral
	signal data_IP2BUS	: std_logic_vector(31 downto 0);	-- Data from this peripheral to the bus

	signal chipSel			: std_logic;

	signal testReg			: std_logic_vector(31 downto 0);
	signal lfsr				: std_logic_vector(31 downto 0);

	function reverseVector (a: in std_logic_vector) return std_logic_vector is
		variable result: std_logic_vector(a'reverse_range);
	begin
		for i in a'range loop
			result((a'length-1)-i) := a(i);
 		end loop;
		return result;
	end;

begin

Sl_errAck		<= '0';
Sl_retry			<= '0';
Sl_toutSup		<= '0';

-- Create a chipSel signal
-- Set chipSel if OPB_select is high and the address is within the range of this peripheral
chipSel			<= '1' when (OPB_select='1' and OPB_ABus(0 to 25)=C_BASEADDR(0 to 25)) else '0';

Sl_DBus			<= reverseVector(data_IP2BUS);	-- Use the reverseVector function to change vector direction
data_BUS2IP		<= reverseVector(OPB_DBus);

ReadControl : process(OPB_RNW, chipSel, OPB_ABus, lfsr)
	variable data_IP2BUS_var	: std_logic_vector(31 downto 0);
	variable Sl_xferAck_var		: std_logic;
begin

	data_IP2BUS_var	:= (others => '0');	-- Make the default data output all zeros
	Sl_xferAck_var		:= '0';					-- Make the default acknowledge 0

	-- Read
	if (OPB_RNW='1' and chipSel='1') then
		case OPB_ABus(26 to 29) is
			when x"0"	=> data_IP2BUS_var := testReg;	-- Reg 0
			when x"1"	=> data_IP2BUS_var := lfsr;		-- Reg 1

			when others	=> null;
		end case;
		Sl_xferAck_var	:= '1';	-- Acknowledge this read
	end if;

	data_IP2BUS <= data_IP2BUS_var;
	Sl_xferAck	<= Sl_xferAck_var;

end process ReadControl;

WriteControl : process(OPB_Clk)
	variable lfsrIn : std_logic;
begin
	if (OPB_Clk'event and OPB_Clk='1') then
		if (OPB_Rst='1') then
			testReg	<= (others => '0');
			lfsr		<= (others => '0');

		elsif (OPB_RNW='0' and chipSel='1') then
			case OPB_ABus(26 to 29) is
				when x"0"	=> testReg	<= data_BUS2IP;		-- Reg 0
				when x"1"	=> lfsr		<= data_BUS2IP;		-- Reg 1
				when others	=> null;
			end case;

		else
			lfsrIn := lfsr(TAP0_POS) xor lfsr(TAP1_POS) xor lfsr(TAP2_POS) xor lfsr(TAP3_POS);
			lfsr <= lfsr(30 downto 0) & lfsrIn;

		end if;
	end if;
end process WriteControl;

end IMP;
